@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2010  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,7 @@
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -50,6 +51,10 @@
 
 #include "hciattach.h"
 
+#ifdef NEED_PPOLL
+#include "ppoll.h"
+#endif
+
 struct uart_t {
 	char *type;
 	int  m_id;
@@ -141,10 +146,10 @@ int set_speed(int fd, struct termios *ti, int speed)
 	return tcsetattr(fd, TCSANOW, ti);
 }
 
-/* 
+/*
  * Read an HCI event from the given file descriptor.
  */
-int read_hci_event(int fd, unsigned char* buf, int size) 
+int read_hci_event(int fd, unsigned char* buf, int size)
 {
 	int remain, r;
 	int count = 0;
@@ -172,9 +177,9 @@ int read_hci_event(int fd, unsigned char* buf, int size)
 	}
 
 	/* Now we read the parameters. */
-	if (buf[2] < (size - 3)) 
+	if (buf[2] < (size - 3))
 		remain = buf[2];
-	else 
+	else
 		remain = size - 3;
 
 	while ((count - 3) < remain) {
@@ -187,8 +192,8 @@ int read_hci_event(int fd, unsigned char* buf, int size)
 	return count;
 }
 
-/* 
- * Ericsson specific initialization 
+/*
+ * Ericsson specific initialization
  */
 static int ericsson(int fd, struct uart_t *u, struct termios *ti)
 {
@@ -242,8 +247,8 @@ static int ericsson(int fd, struct uart_t *u, struct termios *ti)
 	return 0;
 }
 
-/* 
- * Digianswer specific initialization 
+/*
+ * Digianswer specific initialization
  */
 static int digi(int fd, struct uart_t *u, struct termios *ti)
 {
@@ -301,7 +306,7 @@ static int read_check(int fd, void *buf, int count)
 	do {
 		res = read(fd, buf, count);
 		if (res != -1) {
-			buf += res; 
+			buf += res;
 			count -= res;
 		}
 	} while (count && (errno == 0 || errno == EINTR));
@@ -315,15 +320,15 @@ static int read_check(int fd, void *buf, int count)
 /*
  * BCSP specific initialization
  */
-int serial_fd;
+static int serial_fd;
+static int bcsp_max_retries = 10;
 
 static void bcsp_tshy_sig_alarm(int sig)
 {
-	static int retries=0;
 	unsigned char bcsp_sync_pkt[10] = {0xc0,0x00,0x41,0x00,0xbe,0xda,0xdc,0xed,0xed,0xc0};
-	int len;
+	int len, retries = 0;
 
-	if (retries < 10) {
+	if (retries < bcsp_max_retries) {
 		retries++;
 		len = write(serial_fd, &bcsp_sync_pkt, 10);
 		alarm(1);
@@ -337,11 +342,10 @@ static void bcsp_tshy_sig_alarm(int sig)
 
 static void bcsp_tconf_sig_alarm(int sig)
 {
-	static int retries=0;
 	unsigned char bcsp_conf_pkt[10] = {0xc0,0x00,0x41,0x00,0xbe,0xad,0xef,0xac,0xed,0xc0};
-	int len;
+	int len, retries = 0;
 
-	if (retries < 10){
+	if (retries < bcsp_max_retries){
 		retries++;
 		len = write(serial_fd, &bcsp_conf_pkt, 10);
 		alarm(1);
@@ -476,8 +480,8 @@ static int bcsp(int fd, struct uart_t *u, struct termios *ti)
 	return 0;
 }
 
-/* 
- * CSR specific initialization 
+/*
+ * CSR specific initialization
  * Inspired strongly by code in OpenBT and experimentations with Brainboxes
  * Pcmcia card.
  * Jean Tourrilhes <jt@hpl.hp.com> - 14.11.01
@@ -540,7 +544,7 @@ static int csr(int fd, struct uart_t *u, struct termios *ti)
 			return -1;
 		}
 
-	/* Event code 0xFF is for vendor-specific events, which is 
+	/* Event code 0xFF is for vendor-specific events, which is
 	 * what we're looking for. */
 	} while (resp[1] != 0xFF);
 
@@ -556,7 +560,7 @@ static int csr(int fd, struct uart_t *u, struct termios *ti)
 	}
 #endif
 	/* Display that to user */
-	fprintf(stderr, "CSR build ID 0x%02X-0x%02X\n", 
+	fprintf(stderr, "CSR build ID 0x%02X-0x%02X\n",
 		resp[15] & 0xFF, resp[14] & 0xFF);
 
 	/* Try to read the current speed of the CSR chip */
@@ -584,7 +588,7 @@ static int csr(int fd, struct uart_t *u, struct termios *ti)
 			return -1;
 		}
 
-	/* Event code 0xFF is for vendor-specific events, which is 
+	/* Event code 0xFF is for vendor-specific events, which is
 	 * what we're looking for. */
 	} while (resp[1] != 0xFF);
 
@@ -598,7 +602,7 @@ static int csr(int fd, struct uart_t *u, struct termios *ti)
 #endif
 
 	if (u->speed > 1500000) {
-		fprintf(stderr, "Speed %d too high. Remaining at %d baud\n", 
+		fprintf(stderr, "Speed %d too high. Remaining at %d baud\n",
 			u->speed, u->init_speed);
 		u->speed = u->init_speed;
 	} else if (u->speed != 57600 && uart_speed(u->speed) == B57600) {
@@ -648,8 +652,8 @@ static int csr(int fd, struct uart_t *u, struct termios *ti)
 	return 0;
 }
 
-/* 
- * Silicon Wave specific initialization 
+/*
+ * Silicon Wave specific initialization
  * Thomas Moser <thomas.moser@tmoser.ch>
  */
 static int swave(int fd, struct uart_t *u, struct termios *ti)
@@ -665,7 +669,7 @@ static int swave(int fd, struct uart_t *u, struct termios *ti)
 	// Subcommand", e.g. "soft reset" to make the changes effective.
 
 	cmd[0] = HCI_COMMAND_PKT;	// it's a command packet
-	cmd[1] = 0x0B;			// OCF 0x0B	= param access set	
+	cmd[1] = 0x0B;			// OCF 0x0B	= param access set
 	cmd[2] = 0xfc;			// OGF bx111111 = vendor specific
 	cmd[3] = 0x06;			// 6 bytes of data following
 	cmd[4] = 0x01;			// param sub command
@@ -699,9 +703,9 @@ static int swave(int fd, struct uart_t *u, struct termios *ti)
 		return -1;
 	}
 
-	// We should wait for a "GET Event" to confirm the success of 
-	// the baud rate setting. Wait some time before reading. Better:  
-	// read with timeout, parse data 
+	// We should wait for a "GET Event" to confirm the success of
+	// the baud rate setting. Wait some time before reading. Better:
+	// read with timeout, parse data
 	// until correct answer, else error handling ... todo ...
 
 	nanosleep(&tm, NULL);
@@ -745,7 +749,7 @@ static int swave(int fd, struct uart_t *u, struct termios *ti)
 	// now the uart baud rate on the silicon wave module is set and effective.
 	// change our own baud rate as well. Then there is a reset event comming in
  	// on the *new* baud rate. This is *undocumented*! The packet looks like this:
-	// 04 FF 01 0B (which would make that a confirmation of 0x0B = "Param 
+	// 04 FF 01 0B (which would make that a confirmation of 0x0B = "Param
 	// subcommand class". So: change to new baud rate, read with timeout, parse
 	// data, error handling. BTW: all param access in Silicon Wave is done this way.
 	// Maybe this code would belong in a seperate file, or at least code reuse...
@@ -1182,6 +1186,7 @@ int main(int argc, char *argv[])
 	pid_t pid;
 	struct sigaction sa;
 	struct pollfd p;
+	sigset_t sigs;
 	char dev[PATH_MAX];
 
 	detach = 1;
@@ -1254,7 +1259,7 @@ int main(int argc, char *argv[])
 				fprintf(stderr, "Unknown device type or id\n");
 				exit(1);
 			}
-			
+
 			break;
 
 		case 2:
@@ -1291,13 +1296,16 @@ int main(int argc, char *argv[])
 
 	/* 10 seconds should be enough for initialization */
 	alarm(to);
+	bcsp_max_retries = to;
 
 	n = init_uart(dev, u, send_break);
 	if (n < 0) {
-		perror("Can't initialize device"); 
+		perror("Can't initialize device");
 		exit(1);
 	}
 
+	printf("Device setup complete\n");
+
 	alarm(0);
 
 	memset(&sa, 0, sizeof(sa));
@@ -1328,9 +1336,16 @@ int main(int argc, char *argv[])
 	p.fd = n;
 	p.events = POLLERR | POLLHUP;
 
+	sigfillset(&sigs);
+	sigdelset(&sigs, SIGCHLD);
+	sigdelset(&sigs, SIGPIPE);
+	sigdelset(&sigs, SIGTERM);
+	sigdelset(&sigs, SIGINT);
+	sigdelset(&sigs, SIGHUP);
+
 	while (!__io_canceled) {
 		p.revents = 0;
-		err = poll(&p, 1, 500);
+		err = ppoll(&p, 1, NULL, &sigs);
 		if (err < 0 && errno == EINTR)
 			continue;
 		if (err)
